@@ -4,7 +4,7 @@
  * Home page of code is: http://smartmontools.sourceforge.net
  * Address of support mailing list: smartmontools-support@lists.sourceforge.net
  *
- * Copyright (C) 2003 Philip Williams, Bruce Allen
+ * Copyright (C) 2003-4 Philip Williams, Bruce Allen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #include "utility.h"
 #include "config.h"
 
-const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.58 2003/11/26 20:41:56 pjwilliams Exp $"
+const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.85 2004/02/11 02:18:21 ballen4705 Exp $"
                                 ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID UTILITY_H_CVSID;
 
 #define MODEL_STRING_LENGTH                         40
@@ -56,11 +56,15 @@ extern long long bytes;
 // to hold onto exit code for atexit routine
 extern int exitstatus;
 
-// These two are common to several models.
+// These three are common to several models.
 const unsigned char vendoropts_9_minutes[][2] = {
   PRESET_9_MINUTES,
   {0,0}
 };
+const unsigned char vendoropts_9_halfminutes[][2] = {
+  PRESET_9_HALFMINUTES,
+  {0,0}
+};
 const unsigned char vendoropts_9_seconds[][2] = {
   PRESET_9_SECONDS,
   {0,0}
@@ -100,7 +104,7 @@ const unsigned char vendoropts_Samsung_SV1204H[][2] = {
   {0,0}
 };
 
-const unsigned char vendoropts_Hitachi_DK23EA[][2] = {
+const unsigned char vendoropts_Hitachi_DK23XX[][2] = {
   PRESET_9_MINUTES,
   PRESET_193_LOADUNLOAD,
   {0,0}
@@ -140,7 +144,12 @@ const drivesettings knowndrives[] = {
       "Please see http://www.geocities.com/dtla_update/index.html#rel",
     NULL, NULL, NULL
   },
-  { // IBM Deskstar 40GV & 75GXP series
+  { // IBM Deskstar 40GV & 75GXP series (TXAOA5AA firmware)
+    "(IBM-)?DTLA-30[57]0[123467][05]$",
+    "^TXAOA5AA$",
+    NULL, NULL, NULL, NULL
+  },
+  { // IBM Deskstar 40GV & 75GXP series (all other firmware)
     "(IBM-)?DTLA-30[57]0[123467][05]$",
     ".*",
     "IBM Deskstar 40GV and 75GXP drives may need upgraded SMART firmware.\n"
@@ -148,7 +157,14 @@ const drivesettings knowndrives[] = {
     NULL, NULL, NULL
   },
   { // Fujitsu MPD and MPE series
-    "^FUJITSU MP[DE]....A[HT]$",
+    "^FUJITSU MP[DE]....A[HTE]$",
+    ".*",
+    NULL,
+    vendoropts_9_seconds,
+    NULL, NULL
+  },
+  { // Fujitsu MPF series
+    "^FUJITSU MPF3(102A[HT]|153A[HT]|204A[HT])$",
     ".*",
     NULL,
     vendoropts_9_seconds,
@@ -199,6 +215,14 @@ const drivesettings knowndrives[] = {
     specialpurpose_reverse_samsung2,
     same_as_minus_F2
   },
+  { // SAMSUNG SP0802N (TK100-23 firmware)
+    "^SAMSUNG SP0802N$",
+    "^TK100-23$",
+    NULL,
+    vendoropts_Samsung_SV4012H,
+    specialpurpose_reverse_samsung2,
+    same_as_minus_F2
+  },
   { // Any other Samsung disk with *-23 firmware
     "^SAMSUNG .*$",
     ".*-23$",
@@ -262,14 +286,34 @@ const drivesettings knowndrives[] = {
     NULL,
     NULL
   },
+  { // All Samsung drives with '.*-25' firmware
+    "^SAMSUNG.*",
+    ".*-25$",
+    "May need -F samsung2 enabled; see manual page for description of -F.\n",
+    vendoropts_9_halfminutes,
+    NULL, NULL
+  },
+  { // SAMSUNG SP40A2H with RR100-07 firmware
+    "^SAMSUNG SP40A2H$",
+    "^RR100-07$",
+    NULL,
+    vendoropts_9_halfminutes,
+    specialpurpose_reverse_samsung,
+    same_as_minus_F
+  },
   { // Samsung ALL OTHER DRIVES
     "^SAMSUNG.*",
     ".*",
     "Contact developers at " PACKAGE_BUGREPORT "; may need -F samsung[2] enabled.\n",
     NULL, NULL, NULL
   },
-  { // Maxtor 6L080J4 and 4K080H4
-    "^MAXTOR (6L080J4|4K080H4)$",
+  { // Maxtor DiamondMax Plus D740X family
+    "^MAXTOR 6L0(20[JL]1|40[JL]2|60[JL]3|80[JL]4)$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Maxtor 4K080H4
+    "^MAXTOR 4K080H4$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -287,8 +331,15 @@ const drivesettings knowndrives[] = {
     vendoropts_9_minutes,
     NULL, NULL
   },
-  { // Maxtor 4W040H3
-    "^Maxtor 4W040H3$",
+  { // Maxtor DiamondMax 4320 family
+    "^Maxtor (91728D8|91512D7|91303D6|91080D5|90845D4|90645D3|90648D4|90432D2)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax 20 VL family
+    "^Maxtor (94091U8|93071U6|92561U5|92041U4|91731U4|91531U3|91361U3|91021U2|90841U2|90651U2)$",
     ".*",
     NULL,
     vendoropts_9_minutes,
@@ -301,6 +352,48 @@ const drivesettings knowndrives[] = {
     vendoropts_9_minutes,
     NULL, NULL
   },
+  { // Maxtor DiamondMax 36 family
+    "^Maxtor (93652U8|92739U6|91826U4|91369U3|90913U2|90845U2|90435U1)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax Plus 40 series
+    "^Maxtor (54098H8|53073H6|52732H6|52049H4|51536H3|51369H3|51024H2)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax Plus 60 family
+    "^Maxtor 5T0(60H6|40H4|30H3|20H2|10H1)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax 80 family
+    "^Maxtor (98196H8|96147H6)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax 536DX family
+    "^Maxtor 4W(100H6|080H6|060H4|040H3|030H2)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax Plus 8 family
+    "^Maxtor 6E0[234]0L0$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
   { // Maxtor DiamondMax Plus 9 family
     "^Maxtor 6Y((060|080|120|160)L0|(080|120|160|200)P0|(060|080|120|160|200)M0)$",
     ".*",
@@ -308,37 +401,40 @@ const drivesettings knowndrives[] = {
     vendoropts_9_minutes,
     NULL, NULL
   },
-  { // HITACHI_DK23BA-20
-    "^HITACHI_DK23BA-20$",
+  { // HITACHI Travelstar DK23XX series
+    "^HITACHI_DK23..-..$",
     ".*",
     NULL,
-    vendoropts_9_minutes,
+    vendoropts_Hitachi_DK23XX,
     NULL, NULL
   },
-  { // HITACHI_DK23EA-30
-    "^HITACHI_DK23EA-30$",
-    ".*",
-    NULL,
-    vendoropts_Hitachi_DK23EA,
-    NULL, NULL
-  },
-  { // IBM-DARA-212000
-    "^IBM-DARA-212000$",
+  { // IBM Travelstar 25GS, 18GT, and 12GN family
+    "^IBM-DARA-2(25|18|15|12|09|06)000$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
-  { // IBM GXP-180
-    "^IC35L120AVV207-[01]$",
+  { // IBM Travelstar 32GH, 30GT, and 20GN family
+    "^IBM-DJSA-2(32|30|20|10|05)$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
+  { // IBM/Hitachi Travelstar 60GH and 40GN family
+    "^IC25(T060ATC[SX]05|N0[4321]0ATC[SX]04)-.$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
+  { // IBM/Hitachi Deskstar 120GXP family
+    "^IC35L((020|040|060|080|120)AVVA|0[24]0AVVN)07-[01]$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // IBM/Hitachi Deskstar GXP-180 family 
+    "^IC35L(030|060|090|120|180)AVV207-[01]$",
     ".*", 
     NULL, NULL, NULL, NULL 
   },
-  { // IBM Deskstar 120GXP 60GB [Phil -- use for testing]
-    "^IC35L060AVVA07-[01]$",
-    ".*",
-    NULL, NULL, NULL, NULL,
-  },
-  { // IBM Deskstar 120GXP 40GB
-    "^IC35L040AVVN07-0$",
+  { // IBM Travelstar 14GS
+    "^IBM-DCYA-214000$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -350,35 +446,55 @@ const drivesettings knowndrives[] = {
   { // TOSHIBA MK6021GAS [Bruce -- use for testing on laptop]
     "^TOSHIBA MK6021GAS$",
     ".*",
-    NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL
+  },
+  { // TOSHIBA MK4019GAX/MK4019GAXB
+    "^TOSHIBA MK4019GAXB?$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // TOSHIBA MK6409MAV
+    "^TOSHIBA MK6409MAV$",
+    ".*",
+    NULL, NULL, NULL, NULL
   },
   { // Seagate Barracuda U Series 20410
     "^ST320410A$",
     ".*",
-    NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL
+  },
+  { // Seagate U Series 5 20413
+    "^ST320413A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Seagate Barracuda ATA II family
+    "^ST3(3063|2042|1532|1021)0A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Seagate Barracuda ATA IV family
+    "^ST3(20011|40016|60021|80021)A$",
+    ".*",
+    NULL, NULL, NULL, NULL
   },
   { // Seagate Barracuda ATA V family
     "^ST3(12002(3A|4A|9A|3AS)|800(23A|15A|23AS)|60(015A|210A)|40017A)$",
     ".*",
-    NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL
   },
-  { // Seagate Barracuda 7200.7 family
-    "^(ST340014A|ST380011A|ST3120022A|ST3160021A)$",
+  { // Seagate Barracuda 7200.7 and 7200.7 Plus family
+    "^ST3(200822AS?|16002[13]AS?|12002[26]AS?|8001[13]AS?|60014A|40014AS?)$",
     ".*",
-    NULL, NULL, NULL, NULL,
-  },
-  { // Seagate Barracuda 7200.7 Plus family
-    "^(ST3120026A|ST3160023A|ST3200822A)$",
-    ".*",
-    NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL
   },
   { // Western Digital Protege WD400EB
   /* Western Digital drives with this comment all appear to use Attribute 9 in
    * a  non-standard manner.  These entries may need to be updated when it
    * is understood exactly how Attribute 9 should be interpreted. */
-    "^WDC WD400EB-00CPF0$",
+    "^WDC WD400EB-.*$",
     ".*",
-    NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL
   },
   { // Western Digital Caviar family
   /* Western Digital drives with this comment all appear to use Attribute 9 in
@@ -386,7 +502,31 @@ const drivesettings knowndrives[] = {
    * is understood exactly how Attribute 9 should be interpreted. */
     "^WDC WD(2|3|4|6|8|10|12|16|18|20|25)00BB-.*$",
     ".*",
-    NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL
+  },
+  { // Western Digital Caviar WDxxxAB series
+  /* Western Digital drives with this comment all appear to use Attribute 9 in
+   * a  non-standard manner.  These entries may need to be updated when it
+   * is understood exactly how Attribute 9 should be interpreted. */
+    "^WDC WD(3|4|6)00AB-.*$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Western Digital Caviar WDxxxAA series
+  /* Western Digital drives with this comment all appear to use Attribute 9 in
+   * a  non-standard manner.  These entries may need to be updated when it
+   * is understood exactly how Attribute 9 should be interpreted. */
+    "^WDC WD(64|84|102|136|205|307)AA$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Western Digital Caviar WDxxxBA series
+  /* Western Digital drives with this comment all appear to use Attribute 9 in
+   * a  non-standard manner.  These entries may need to be updated when it
+   * is understood exactly how Attribute 9 should be interpreted. */
+    "^WDC WD(102|136|153|205)BA$",
+    ".*",
+    NULL, NULL, NULL, NULL
   },
   { // Western Digital Caviar SE family
   /* Western Digital drives with this comment all appear to use Attribute 9 in
@@ -394,12 +534,32 @@ const drivesettings knowndrives[] = {
    * is understood exactly how Attribute 9 should be interpreted. */
     "^WDC WD(4|6|8|10|12|16|18|20|25)00JB-.*$",
     ".*",
-    NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL
   },
   { // Western Digital Caviar AC38400
     "^WDC AC38400L$",
     ".*",
-    NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL
+  },
+  { // Western Digital Caviar AC23200L
+    "^WDC AC23200L$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // QUANTUM FIREBALLlct15 20
+    "^QUANTUM FIREBALLlct15 20$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // QUANTUM FIREBALLlct20 20
+    "^QUANTUM FIREBALLlct20 20$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // QUANTUM FIREBALL CX10.2A
+    "^QUANTUM FIREBALL CX10.2A$",
+    ".*",
+    NULL, NULL, NULL, NULL
   },
   /*------------------------------------------------------------
    *  End of table.  Do not add entries below this marker.
@@ -458,7 +618,7 @@ void showonepreset(const drivesettings *drivetable){
   // Basic error check
   if (!drivetable || !drivetable->modelregexp){
     pout("Null known drive table pointer. Please report\n"
-	 "this error to smartmontools developers at " PACKAGE_BUGREPORT ".\n");
+         "this error to smartmontools developers at " PACKAGE_BUGREPORT ".\n");
     return;
   }
   
@@ -497,8 +657,8 @@ void showonepreset(const drivesettings *drivetable){
   if (drivetable->specialpurpose){
     pout("%-*s ", TABLEPRINTWIDTH, "OTHER PRESETS:");
     pout("%s\n", drivetable->functiondesc ?
-	 drivetable->functiondesc : "A special purpose function "
-	 "is defined for this drive"); 
+         drivetable->functiondesc : "A special purpose function "
+         "is defined for this drive"); 
   }
   
   // Print any special warnings
@@ -519,6 +679,8 @@ void showallpresets(void){
     showonepreset(&knowndrives[i]);
     pout("\n");
   }
+  pout("For information about adding a drive to the database see the FAQ on the\n");
+  pout("smartmontools home page: " PACKAGE_HOMEPAGE "\n");
   return;
 }
 
@@ -535,11 +697,11 @@ void showpresets(const struct ata_identify_device *drive){
   if ((i = lookupdrive(model, firmware)) < 0) {
     // no matches found
     pout("No presets are defined for this drive.  Its identity strings:\n"
-	 "MODEL:    %s\n"
-	 "FIRMWARE: %s\n"
-	 "do not match any of the known regular expressions.\n"
-	 "Use -P showall to list all known regular expressions.\n",
-	 model, firmware);
+         "MODEL:    %s\n"
+         "FIRMWARE: %s\n"
+         "do not match any of the known regular expressions.\n"
+         "Use -P showall to list all known regular expressions.\n",
+         model, firmware);
     return;
   }
   
