@@ -3,7 +3,7 @@
  *
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2003 Eduard Martinescu <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2003-4 Eduard Martinescu <smartmontools-support@lists.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +39,9 @@
 #include "utility.h"
 #include "os_freebsd.h"
 
-const char *os_XXXX_c_cvsid="$Id: os_freebsd.c,v 1.23 2003/11/06 04:27:15 arvoreen Exp $" \
+static const char *filenameandversion="$Id: os_freebsd.c,v 1.31 2004/01/31 01:39:55 arvoreen Exp $";
+
+const char *os_XXXX_c_cvsid="$Id: os_freebsd.c,v 1.31 2004/01/31 01:39:55 arvoreen Exp $" \
 ATACMDS_H_CVSID CONFIG_H_CVSID OS_XXXX_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID;
 
 // to hold onto exit code for atexit routine
@@ -52,6 +54,34 @@ struct freebsd_dev_channel *devicetable[FREEBSD_MAXDEV];
 // forward declaration
 static int parse_ata_chan_dev(const char * dev_name, struct freebsd_dev_channel *ch);
 
+// print examples for smartctl
+void print_smartctl_examples(){
+  printf("=================================================== SMARTCTL EXAMPLES =====\n\n");
+#ifdef HAVE_GETOPT_LONG
+  printf(
+         "  smartctl -a /dev/ad0                       (Prints all SMART information)\n\n"
+         "  smartctl --smart=on --offlineauto=on --saveauto=on /dev/ad0\n"
+         "                                              (Enables SMART on first disk)\n\n"
+         "  smartctl -t long /dev/ad0              (Executes extended disk self-test)\n\n"
+         "  smartctl --attributes --log=selftest --quietmode=errorsonly /dev/ad0\n"
+         "                                      (Prints Self-Test & Attribute errors)\n"
+//         "  smartctl -a -device=3ware,2 /dev/sda\n"
+//         "          (Prints all SMART info for 3rd ATA disk on 3ware RAID controller)\n"
+         );
+#else
+  printf(
+         "  smartctl -a /dev/ad0                       (Prints all SMART information)\n"
+         "  smartctl -s on -o on -S on /dev/ad0         (Enables SMART on first disk)\n"
+         "  smartctl -t long /dev/ad0              (Executes extended disk self-test)\n"
+         "  smartctl -A -l selftest -q errorsonly /dev/ad0\n"
+         "                                      (Prints Self-Test & Attribute errors)\n"
+//         "  smartctl -a -d 3ware,2 /dev/sda\n"
+//         "          (Prints all SMART info for 3rd ATA disk on 3ware RAID controller)\n"
+         );
+#endif
+  return;
+}
+
 // Like open().  Return positive integer handle, used by functions below only.  mode=="ATA" or "SCSI".
 int deviceopen (const char* dev, char* mode) {
   struct freebsd_dev_channel *fdchan;
@@ -84,7 +114,7 @@ int deviceopen (const char* dev, char* mode) {
 
   if (parse_ok == GUESS_DEVTYPE_ATA) {
     if ((fdchan->atacommand = open("/dev/ata",O_RDWR))<0) {
-      int myerror = errno;	//preserve across free call
+      int myerror = errno;      //preserve across free call
       free (fdchan);
       errno = myerror;
       return -1;
@@ -159,7 +189,7 @@ void printwarning(int msgNo, const char* extra) {
   static const char* message[]={
     "The SMART RETURN STATUS return value (smartmontools -H option/Directive)\n can not be retrieved with this version of ATAng, please do not rely on this value\n",
     
-    "Error SMART Status command failed\nPlease get assistance from \n" PROJECTHOME "\nRegister values returned from SMART Status command are:\n",
+    "Error SMART Status command failed\nPlease get assistance from \n" PACKAGE_HOMEPAGE "\nRegister values returned from SMART Status command are:\n",
     
     PACKAGE_STRING " does not currentlly support TWE devices (3ware Escalade)\n",
     
@@ -171,7 +201,7 @@ void printwarning(int msgNo, const char* extra) {
       printed[msgNo] = 1;
       pout("%s", message[msgNo]);
       if (extra)
-	pout("%s",extra);
+        pout("%s",extra);
     }
   }
   return;
@@ -203,11 +233,11 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
   iocmd.channel=con->channel;
   iocmd.device=con->device;
 
-  iocmd.u.request.u.ata.command=WIN_SMART;
+  iocmd.u.request.u.ata.command=ATA_SMART_CMD;
   iocmd.u.request.timeout=600;
   switch (command){
   case READ_VALUES:
-    iocmd.u.request.u.ata.feature=SMART_READ_VALUES;
+    iocmd.u.request.u.ata.feature=ATA_SMART_READ_VALUES;
     iocmd.u.request.u.ata.lba=0xc24f<<8;
     iocmd.u.request.flags=ATA_CMD_READ;
     iocmd.u.request.data=buff;
@@ -215,7 +245,7 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
     copydata=1;
     break;
   case READ_THRESHOLDS:
-    iocmd.u.request.u.ata.feature=SMART_READ_THRESHOLDS;
+    iocmd.u.request.u.ata.feature=ATA_SMART_READ_THRESHOLDS;
     iocmd.u.request.u.ata.count=1;
     iocmd.u.request.u.ata.lba=1|(0xc24f<<8);
     iocmd.u.request.flags=ATA_CMD_READ;
@@ -224,7 +254,7 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
     copydata=1;
     break;
   case READ_LOG:
-    iocmd.u.request.u.ata.feature=SMART_READ_LOG_SECTOR;
+    iocmd.u.request.u.ata.feature=ATA_SMART_READ_LOG_SECTOR;
     iocmd.u.request.u.ata.lba=select|(0xc24f<<8);
     iocmd.u.request.u.ata.count=1;
     iocmd.u.request.flags=ATA_CMD_READ;
@@ -233,43 +263,43 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
     copydata=1;
     break;
   case IDENTIFY:
-    iocmd.u.request.u.ata.command=WIN_IDENTIFY;
+    iocmd.u.request.u.ata.command=ATA_IDENTIFY_DEVICE;
     iocmd.u.request.flags=ATA_CMD_READ;
     iocmd.u.request.data=buff;
     iocmd.u.request.count=512;
     copydata=1;
     break;
   case PIDENTIFY:
-    iocmd.u.request.u.ata.command=WIN_PIDENTIFY;
+    iocmd.u.request.u.ata.command=ATA_IDENTIFY_PACKET_DEVICE;
     iocmd.u.request.flags=ATA_CMD_READ;
     iocmd.u.request.data=buff;
     iocmd.u.request.count=512;
     copydata=1;
     break;
   case ENABLE:
-    iocmd.u.request.u.ata.feature=SMART_ENABLE;
+    iocmd.u.request.u.ata.feature=ATA_SMART_ENABLE;
     iocmd.u.request.u.ata.lba=0xc24f<<8;
     iocmd.u.request.flags=ATA_CMD_CONTROL;
     break;
   case DISABLE:
-    iocmd.u.request.u.ata.feature=SMART_DISABLE;
+    iocmd.u.request.u.ata.feature=ATA_SMART_DISABLE;
     iocmd.u.request.u.ata.lba=0xc24f<<8;
     iocmd.u.request.flags=ATA_CMD_CONTROL;
     break;
   case AUTO_OFFLINE:
     // NOTE: According to ATAPI 4 and UP, this command is obsolete
-    iocmd.u.request.u.ata.feature=SMART_AUTO_OFFLINE;
+    iocmd.u.request.u.ata.feature=ATA_SMART_AUTO_OFFLINE;
     iocmd.u.request.u.ata.lba=select|(0xc24f<<8);
     iocmd.u.request.flags=ATA_CMD_CONTROL;
     break;
   case AUTOSAVE:
-    iocmd.u.request.u.ata.feature=SMART_AUTOSAVE;
+    iocmd.u.request.u.ata.feature=ATA_SMART_AUTOSAVE;
     iocmd.u.request.u.ata.count=0xf1;  // to enable autosave
     iocmd.u.request.u.ata.lba=0xc24f<<8;
     iocmd.u.request.flags=ATA_CMD_CONTROL;
     break;
   case IMMEDIATE_OFFLINE:
-    iocmd.u.request.u.ata.feature=SMART_IMMEDIATE_OFFLINE;
+    iocmd.u.request.u.ata.feature=ATA_SMART_IMMEDIATE_OFFLINE;
     iocmd.u.request.u.ata.lba = select|(0xc24f<<8); // put test in sector
     iocmd.u.request.flags=ATA_CMD_CONTROL;
     break;
@@ -277,7 +307,7 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
   case STATUS:
     // this command only says if SMART is working.  It could be
     // replaced with STATUS_CHECK below.
-    iocmd.u.request.u.ata.feature=SMART_STATUS;
+    iocmd.u.request.u.ata.feature=ATA_SMART_STATUS;
     iocmd.u.request.u.ata.lba=0xc24f<<8;
     iocmd.u.request.flags=ATA_CMD_CONTROL;
 #ifdef ATA_CMD_READ_REG
@@ -288,7 +318,7 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
     break;
   default:
     pout("Unrecognized command %d in ata_command_interface()\n"
-	 "Please contact " PACKAGE_BUGREPORT "\n", command);
+         "Please contact " PACKAGE_BUGREPORT "\n", command);
     errno=ENOSYS;
     return -1;
   }
@@ -319,13 +349,13 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
     // We haven't gotten output that makes sense; print out some debugging info
     char buf[512];
     sprintf(buf,"CMD=0x%02x\nFR =0x%02x\nNS =0x%02x\nSC =0x%02x\nCL =0x%02x\nCH =0x%02x\nRETURN =0x%04x\n",
-	    (int)iocmd.u.request.u.ata.command,
-	    (int)iocmd.u.request.u.ata.feature,
-	    (int)iocmd.u.request.u.ata.count,
-	    (int)((iocmd.u.request.u.ata.lba) & 0xff),
-	    (int)((iocmd.u.request.u.ata.lba>>8) & 0xff),
-	    (int)((iocmd.u.request.u.ata.lba>>16) & 0xff),
-	    (int)iocmd.u.request.error);
+            (int)iocmd.u.request.u.ata.command,
+            (int)iocmd.u.request.u.ata.feature,
+            (int)iocmd.u.request.u.ata.count,
+            (int)((iocmd.u.request.u.ata.lba) & 0xff),
+            (int)((iocmd.u.request.u.ata.lba>>8) & 0xff),
+            (int)((iocmd.u.request.u.ata.lba>>16) & 0xff),
+            (int)iocmd.u.request.error);
     printwarning(BAD_SMART,buf);
     return 0;   
   }
@@ -389,18 +419,18 @@ int do_scsi_cmnd_io(int fd, struct scsi_cmnd_io * iop, int report)
 
   // clear out structure, except for header that was filled in for us
   bzero(&(&ccb->ccb_h)[1],
-	sizeof(struct ccb_scsiio) - sizeof(struct ccb_hdr));
+        sizeof(struct ccb_scsiio) - sizeof(struct ccb_hdr));
 
   cam_fill_csio(&ccb->csio,
-		/*retrires*/ 1,
-		/*cbfcnp*/ NULL,
-		/* flags */ (iop->dxfer_dir == DXFER_NONE ? CAM_DIR_NONE :(iop->dxfer_dir == DXFER_FROM_DEVICE ? CAM_DIR_IN : CAM_DIR_OUT)),
-		/* tagaction */ MSG_SIMPLE_Q_TAG,
-		/* dataptr */ iop->dxferp,
-		/* datalen */ iop->dxfer_len,
-		/* senselen */ iop->max_sense_len,
-		/* cdblen */ iop->cmnd_len,
-		/* timout */ iop->timeout);
+                /*retrires*/ 1,
+                /*cbfcnp*/ NULL,
+                /* flags */ (iop->dxfer_dir == DXFER_NONE ? CAM_DIR_NONE :(iop->dxfer_dir == DXFER_FROM_DEVICE ? CAM_DIR_IN : CAM_DIR_OUT)),
+                /* tagaction */ MSG_SIMPLE_Q_TAG,
+                /* dataptr */ iop->dxferp,
+                /* datalen */ iop->dxfer_len,
+                /* senselen */ iop->max_sense_len,
+                /* cdblen */ iop->cmnd_len,
+                /* timout */ iop->timeout);
   memcpy(ccb->csio.cdb_io.cdb_bytes,iop->cmnd,iop->cmnd_len);
 
   if (cam_send_ccb(cam_dev,ccb) < 0) {
@@ -439,7 +469,7 @@ int do_scsi_cmnd_io(int fd, struct scsi_cmnd_io * iop, int report)
     trunc = (iop->dxfer_len > 256) ? 1 : 0;
     
     pout("  Incoming data, len=%d%s:\n", (int)iop->dxfer_len,
-	 (trunc ? " [only first 256 bytes shown]" : ""));
+         (trunc ? " [only first 256 bytes shown]" : ""));
     dStrHex(iop->dxferp, (trunc ? 256 : iop->dxfer_len) , 1);
   }
   return 0;
@@ -454,7 +484,7 @@ int escalade_command_interface(int fd, int disknum, smart_command_set command, i
 
 static int get_ata_channel_unit ( const char* name, int* unit, int* dev) {
 #ifndef ATAREQUEST
-	return -1;
+        return -1;
 #else
   // there is no direct correlation between name 'ad0, ad1, ...' and
   // channel/unit number.  So we need to iterate through the possible
@@ -527,10 +557,10 @@ static int parse_ata_chan_dev(const char * dev_name, struct freebsd_dev_channel
   }
   // form /dev/ad* or ad*
   if (!strncmp(fbsd_dev_ata_disk_prefix, dev_name,
-	       strlen(fbsd_dev_ata_disk_prefix))) {
+               strlen(fbsd_dev_ata_disk_prefix))) {
     if (chan != NULL) {
       if (get_ata_channel_unit(dev_name,&(chan->channel),&(chan->device))<0) {
-	return GUESS_DEVTYPE_DONT_KNOW;
+        return GUESS_DEVTYPE_DONT_KNOW;
       }
     }
     return GUESS_DEVTYPE_ATA;
@@ -538,22 +568,22 @@ static int parse_ata_chan_dev(const char * dev_name, struct freebsd_dev_channel
   
   // form /dev/da* or da*
   if (!strncmp(fbsd_dev_scsi_disk_plus, dev_name,
-	       strlen(fbsd_dev_scsi_disk_plus)))
+               strlen(fbsd_dev_scsi_disk_plus)))
     goto handlescsi;
 
   // form /dev/sa* or sa*
   if (!strncmp(fbsd_dev_scsi_tape1, dev_name,
-	      strlen(fbsd_dev_scsi_tape1)))
+              strlen(fbsd_dev_scsi_tape1)))
     goto handlescsi;
 
   // form /dev/nsa* or nsa*
   if (!strncmp(fbsd_dev_scsi_tape2, dev_name,
-	      strlen(fbsd_dev_scsi_tape2)))
+              strlen(fbsd_dev_scsi_tape2)))
     goto handlescsi;
 
   // form /dev/esa* or esa*
   if (!strncmp(fbsd_dev_scsi_tape3, dev_name,
-	      strlen(fbsd_dev_scsi_tape3)))
+              strlen(fbsd_dev_scsi_tape3)))
     goto handlescsi;
   
   // we failed to recognize any of the forms
@@ -578,8 +608,6 @@ int guess_device_type (const char* dev_name) {
 // global variable holding byte count of allocated memory
 extern long long bytes;
 
-void *FreeNonZero(void* address, int size,int whatline,char* file);
-
 // we are going to take advantage of the fact that FreeBSD's devfs will only
 // have device entries for devices that exist.  So if we get the equivilent of
 // ls /dev/ad?, we have all the ATA devices on the system
@@ -595,10 +623,11 @@ int get_dev_names(char*** names, const char* prefix) {
   int n = 0;
   char** mp;
   int retglob,lim;
-  glob_t globbuf={0};
+  glob_t globbuf;
   int i;
   char pattern1[128],pattern2[128];
 
+  bzero(&globbuf,sizeof(globbuf));
   // in case of non-clean exit
   *names=NULL;
 
@@ -617,18 +646,18 @@ int get_dev_names(char*** names, const char* prefix) {
     // glob failed
     if (retglob==GLOB_NOSPACE)
       pout("glob(3) ran out of memory matching patterns (%s),(%s)\n",
-	   pattern1, pattern2);
+           pattern1, pattern2);
     else if (retglob==GLOB_ABORTED)
       pout("glob(3) aborted matching patterns (%s),(%s)\n",
-	   pattern1, pattern2);
+           pattern1, pattern2);
     else if (retglob==GLOB_NOMATCH) {
       pout("glob(3) found no matches for patterns (%s),(%s)\n",
-	   pattern1, pattern2);
+           pattern1, pattern2);
       retval = 0;
     }
     else if (retglob)
       pout("Unexplained error in glob(3) of patterns (%s),(%s)\n",
-	   pattern1, pattern2);
+           pattern1, pattern2);
     
     //  Free memory and return
     globfree(&globbuf);
@@ -641,8 +670,8 @@ int get_dev_names(char*** names, const char* prefix) {
   lim = globbuf.gl_pathc < MAX_NUM_DEV ? globbuf.gl_pathc : MAX_NUM_DEV;
   if (lim < globbuf.gl_pathc)
     pout("glob(3) found %d > MAX=%d devices matching patterns (%s),(%s): ignoring %d paths\n", 
-	 globbuf.gl_pathc, MAX_NUM_DEV, pattern1,pattern2,
-	 globbuf.gl_pathc-MAX_NUM_DEV);
+         globbuf.gl_pathc, MAX_NUM_DEV, pattern1,pattern2,
+         globbuf.gl_pathc-MAX_NUM_DEV);
   
   // allocate space for up to lim number of ATA devices
   if (!(mp =  (char **)calloc(lim, sizeof(char*)))){
@@ -657,7 +686,7 @@ int get_dev_names(char*** names, const char* prefix) {
     // the pattern itself will be added to path list..
     // so ignore any paths that have the ']' from pattern
     if (strchr(globbuf.gl_pathv[i],']') == NULL)
-      mp[n++] = CustomStrDup(globbuf.gl_pathv[i], 1, __LINE__, __FILE__);
+      mp[n++] = CustomStrDup(globbuf.gl_pathv[i], 1, __LINE__, filenameandversion);
   }
 
   globfree(&globbuf);
