@@ -3,20 +3,20 @@
  * 
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2003 Bruce Allen <smartmontools-support@lists.sourceforge.net>
- * Copyright (C) 2003 Doug Gilbert <dougg@torque.net>
+ * Copyright (C) 2003-4 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2003-4 Doug Gilbert <dougg@torque.net>
  *
  *  Parts of this file are derived from code that was
  *
  *  Written By: Adam Radford <linux@3ware.com>
  *  Modifications By: Joel Jacobson <linux@3ware.com>
- *  		     Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+ *                   Arnaldo Carvalho de Melo <acme@conectiva.com.br>
  *                    Brad Strand <linux@3ware.com>
  *
  *  Copyright (C) 1999-2003 3ware Inc.
  *
- *  Kernel compatablity By:	Andre Hedrick <andre@suse.com>
- *  Non-Copyright (C) 2000	Andre Hedrick <andre@suse.com>
+ *  Kernel compatablity By:     Andre Hedrick <andre@suse.com>
+ *  Non-Copyright (C) 2000      Andre Hedrick <andre@suse.com>
  *
  * Other ars of this file are derived from code that was
  * 
@@ -53,14 +53,25 @@
 #include <sys/stat.h>
 #include <glob.h>
 
-#include "atacmds.h"
 #include "config.h"
+
+#ifdef HAVE_LINUX_HDREG_H
+#include <linux/hdreg.h>
+#else
+#define HDIO_DRIVE_TASK   0x031e
+#define HDIO_DRIVE_CMD    0x031f
+#define HDIO_GET_IDENTITY 0x030d
+#endif
+
+#include "atacmds.h"
 #include "os_linux.h"
 #include "scsicmds.h"
 #include "smartd.h"
 #include "utility.h"
 
-const char *os_XXXX_c_cvsid="$Id: os_linux.c,v 1.31 2003/11/26 10:34:29 dpgilbert Exp $" \
+static const char *filenameandversion="$Id: os_linux.c,v 1.45 2004/02/08 07:12:40 ballen4705 Exp $";
+
+const char *os_XXXX_c_cvsid="$Id: os_linux.c,v 1.45 2004/02/08 07:12:40 ballen4705 Exp $" \
 ATACMDS_H_CVSID CONFIG_H_CVSID OS_XXXX_H_CVSID SCSICMDS_H_CVSID SMARTD_H_CVSID UTILITY_H_CVSID;
 
 // to hold onto exit code for atexit routine
@@ -69,8 +80,6 @@ extern int exitstatus;
 // global variable holding byte count of allocated memory
 extern long long bytes;
 
-void *FreeNonZero(void* address, int size,int whatline,char* file);
-
 // equivalent to open(path, flags)
 int deviceopen(const char *pathname, char *type){
   if (!strcmp(type,"SCSI")) {
@@ -90,6 +99,34 @@ int deviceclose(int fd){
   return close(fd);
 }
 
+// print examples for smartctl
+void print_smartctl_examples(){
+  printf("=================================================== SMARTCTL EXAMPLES =====\n\n");
+#ifdef HAVE_GETOPT_LONG
+  printf(
+         "  smartctl -a /dev/hda                       (Prints all SMART information)\n\n"
+         "  smartctl --smart=on --offlineauto=on --saveauto=on /dev/hda\n"
+         "                                              (Enables SMART on first disk)\n\n"
+         "  smartctl -t long /dev/hda              (Executes extended disk self-test)\n\n"
+         "  smartctl --attributes --log=selftest --quietmode=errorsonly /dev/hda\n"
+         "                                      (Prints Self-Test & Attribute errors)\n"
+         "  smartctl -a -device=3ware,2 /dev/sda\n"
+         "          (Prints all SMART info for 3rd ATA disk on 3ware RAID controller)\n"
+         );
+#else
+  printf(
+         "  smartctl -a /dev/hda                       (Prints all SMART information)\n"
+         "  smartctl -s on -o on -S on /dev/hda         (Enables SMART on first disk)\n"
+         "  smartctl -t long /dev/hda              (Executes extended disk self-test)\n"
+         "  smartctl -A -l selftest -q errorsonly /dev/hda\n"
+         "                                      (Prints Self-Test & Attribute errors)\n"
+         "  smartctl -a -d 3ware,2 /dev/sda\n"
+         "          (Prints all SMART info for 3rd ATA disk on 3ware RAID controller)\n"
+         );
+#endif
+  return;
+}
+
 
 // we are going to take advantage of the fact that Linux's devfs will only
 // have device entries for devices that exist.  So if we get the equivalent of
@@ -132,7 +169,7 @@ int get_dev_names(char*** names, const char* pattern, const char* name, int max)
   lim = ((int)globbuf.gl_pathc < max) ? (int)globbuf.gl_pathc : max;
   if (lim < (int)globbuf.gl_pathc)
     pout("glob(3) found %d > MAX=%d devices matching pattern %s: ignoring %d paths\n", 
-	 (int)globbuf.gl_pathc, max, pattern, (int)(globbuf.gl_pathc-max));
+         (int)globbuf.gl_pathc, max, pattern, (int)(globbuf.gl_pathc-max));
   
   // allocate space for up to lim number of ATA devices
   if (!(mp =  (char **)calloc(lim, sizeof(char*)))){
@@ -154,28 +191,28 @@ int get_dev_names(char*** names, const char* pattern, const char* name, int max)
     
     // if not a link (or a strange link), keep it
     if (retlink<=0 || retlink>1023)
-      mp[n++] = CustomStrDup(globbuf.gl_pathv[i], 1, __LINE__, __FILE__);
+      mp[n++] = CustomStrDup(globbuf.gl_pathv[i], 1, __LINE__, filenameandversion);
     else {
       // or if it's a link that points to a disc, follow it
       char *p;
       linkbuf[retlink]='\0';
       if ((p=strrchr(linkbuf,'/')) && !strcmp(p+1, "disc"))
-	// This is the branch of the code that gets followed if we are
-	// using devfs WITH traditional compatibility links. In this
-	// case, we add the traditional device name to the list that
-	// is returned.
-	mp[n++] = CustomStrDup(globbuf.gl_pathv[i], 1, __LINE__, __FILE__);
+        // This is the branch of the code that gets followed if we are
+        // using devfs WITH traditional compatibility links. In this
+        // case, we add the traditional device name to the list that
+        // is returned.
+        mp[n++] = CustomStrDup(globbuf.gl_pathv[i], 1, __LINE__, filenameandversion);
       else {
-	// This is the branch of the code that gets followed if we are
-	// using devfs WITHOUT traditional compatibility links.  In
-	// this case, we check that the link to the directory is of
-	// the correct type, and then append "disc" to it.
-	char tmpname[1024]={0};
-	char *type=strcmp(name,"ATA")?"scsi":"ide";
-	if (strstr(linkbuf, type)){
-	  snprintf(tmpname, 1024, "%s/disc", globbuf.gl_pathv[i]);
-	  mp[n++] = CustomStrDup(tmpname, 1, __LINE__, __FILE__);
-	}
+        // This is the branch of the code that gets followed if we are
+        // using devfs WITHOUT traditional compatibility links.  In
+        // this case, we check that the link to the directory is of
+        // the correct type, and then append "disc" to it.
+        char tmpname[1024]={0};
+        char *type=strcmp(name,"ATA")?"scsi":"ide";
+        if (strstr(linkbuf, type)){
+          snprintf(tmpname, 1024, "%s/disc", globbuf.gl_pathv[i]);
+          mp[n++] = CustomStrDup(tmpname, 1, __LINE__, filenameandversion);
+        }
       }
     }
   }
@@ -202,9 +239,9 @@ int make_device_names (char*** devlist, const char* name) {
 #endif
   
   if (!strcmp(name,"SCSI"))
-    retval=get_dev_names(devlist,"/dev/sd[a-z]", name, maxdev=MAXSCSIDEVICES);
+    retval=get_dev_names(devlist,"/dev/sd[a-z]", name, maxdev=26);
   else if (!strcmp(name,"ATA"))
-    retval=get_dev_names(devlist,"/dev/hd[a-t]", name, maxdev=MAXATADEVICES);
+    retval=get_dev_names(devlist,"/dev/hd[a-t]", name, maxdev=20);
   else
     // don't recognize disk type!
     return 0;
@@ -240,8 +277,8 @@ int make_device_names (char*** devlist, const char* name) {
 
 
 // huge value of buffer size needed because HDIO_DRIVE_CMD assumes
-// that buff[3] is the data size.  Since the SMART_AUTOSAVE and
-// SMART_AUTO_OFFLINE use values of 0xf1 and 0xf8 we need the space.
+// that buff[3] is the data size.  Since the ATA_SMART_AUTOSAVE and
+// ATA_SMART_AUTO_OFFLINE use values of 0xf1 and 0xf8 we need the space.
 // Otherwise a 4+512 byte buffer would be enough.
 #define STRANGE_BUFFER_LENGTH (4+512*0xf8)
 
@@ -249,77 +286,85 @@ int ata_command_interface(int device, smart_command_set command, int select, cha
   unsigned char buff[STRANGE_BUFFER_LENGTH];
   int retval, copydata=0;
 
-  // The two linux IOCTL's that we use.  These MUST be consistent with
-  // the same quantities in the kernel include file linux/hdreg.h
-  const int HDIO_DRIVE_TASK = 0x031e;
-  const int HDIO_DRIVE_CMD  = 0x031f;
   const int HDIO_DRIVE_CMD_OFFSET = 4;
 
-  // See struct hd_drive_cmd_hdr in hdreg.h
+  // See struct hd_drive_cmd_hdr in hdreg.h.  Before calling ioctl()
   // buff[0]: ATA COMMAND CODE REGISTER
   // buff[1]: ATA SECTOR NUMBER REGISTER == LBA LOW REGISTER
   // buff[2]: ATA FEATURES REGISTER
   // buff[3]: ATA SECTOR COUNT REGISTER
+
+  // Note that on return:
+  // buff[2] contains the ATA SECTOR COUNT REGISTER
   
   // clear out buff.  Large enough for HDIO_DRIVE_CMD (4+512 bytes)
   memset(buff, 0, STRANGE_BUFFER_LENGTH);
 
-  buff[0]=WIN_SMART;
+  buff[0]=ATA_SMART_CMD;
   switch (command){
+  case CHECK_POWER_MODE:
+    buff[0]=ATA_CHECK_POWER_MODE;
+    copydata=1;
+    break;
   case READ_VALUES:
-    buff[2]=SMART_READ_VALUES;
-    copydata=buff[3]=1;
+    buff[2]=ATA_SMART_READ_VALUES;
+    buff[3]=1;
+    copydata=512;
     break;
   case READ_THRESHOLDS:
-    buff[2]=SMART_READ_THRESHOLDS;
-    copydata=buff[1]=buff[3]=1;
+    buff[2]=ATA_SMART_READ_THRESHOLDS;
+    buff[1]=buff[3]=1;
+    copydata=512;
     break;
   case READ_LOG:
-    buff[2]=SMART_READ_LOG_SECTOR;
+    buff[2]=ATA_SMART_READ_LOG_SECTOR;
     buff[1]=select;
-    copydata=buff[3]=1;
+    buff[3]=1;
+    copydata=512;
     break;
   case IDENTIFY:
-    buff[0]=WIN_IDENTIFY;
-    copydata=buff[3]=1;
+    buff[0]=ATA_IDENTIFY_DEVICE;
+    buff[3]=1;
+    copydata=512;
     break;
   case PIDENTIFY:
-    buff[0]=WIN_PIDENTIFY;
-    copydata=buff[3]=1;
+    buff[0]=ATA_IDENTIFY_PACKET_DEVICE;
+    buff[3]=1;
+    copydata=512;
     break;
   case ENABLE:
-    buff[2]=SMART_ENABLE;
+    buff[2]=ATA_SMART_ENABLE;
     buff[1]=1;
     break;
   case DISABLE:
-    buff[2]=SMART_DISABLE;
+    buff[2]=ATA_SMART_DISABLE;
     buff[1]=1;
     break;
   case STATUS:
     // this command only says if SMART is working.  It could be
     // replaced with STATUS_CHECK below.
-    buff[2]=SMART_STATUS;
+    buff[2]=ATA_SMART_STATUS;
     break;
   case AUTO_OFFLINE:
-    buff[2]=SMART_AUTO_OFFLINE;
+    buff[2]=ATA_SMART_AUTO_OFFLINE;
     buff[3]=select;   // YET NOTE - THIS IS A NON-DATA COMMAND!!
     break;
   case AUTOSAVE:
-    buff[2]=SMART_AUTOSAVE;
+    buff[2]=ATA_SMART_AUTOSAVE;
     buff[3]=select;   // YET NOTE - THIS IS A NON-DATA COMMAND!!
     break;
   case IMMEDIATE_OFFLINE:
-    buff[2]=SMART_IMMEDIATE_OFFLINE;
+    buff[2]=ATA_SMART_IMMEDIATE_OFFLINE;
     buff[1]=select;
     break;
   case STATUS_CHECK:
     // This command uses HDIO_DRIVE_TASK and has different syntax than
     // the other commands.
-    buff[1]=SMART_STATUS;
+    buff[1]=ATA_SMART_STATUS;
     break;
   default:
     pout("Unrecognized command %d in linux_ata_command_interface()\n"
-	 "Please contact " PACKAGE_BUGREPORT "\n", command);
+         "Please contact " PACKAGE_BUGREPORT "\n", command);
     errno=ENOSYS;
     return -1;
   }
@@ -356,7 +401,7 @@ int ata_command_interface(int device, smart_command_set command, int select, cha
     
     // We haven't gotten output that makes sense; print out some debugging info
     syserror("Error SMART Status command failed");
-    pout("Please get assistance from %s\n",PROJECTHOME);
+    pout("Please get assistance from " PACKAGE_HOMEPAGE "\n");
     pout("Register values returned from SMART Status command are:\n");
     pout("CMD=0x%02x\n",(int)buff[0]);
     pout("FR =0x%02x\n",(int)buff[1]);
@@ -386,13 +431,12 @@ int ata_command_interface(int device, smart_command_set command, int select, cha
   // command set, and return 'Command Aborted' to IDENTIFY DEVICE.
   if (command==IDENTIFY || command==PIDENTIFY){
     unsigned char deviceid[512];
-    const int HDIO_GET_IDENTITY=0x030d;
     // check the device identity, as seen when the system was booted
     // or the device was FIRST registered.  This will not be current
     // if the user has subsequently changed some of the parameters. If
     // device is a packet device, swap the command interpretations.
     if (!ioctl(device, HDIO_GET_IDENTITY, deviceid) && (deviceid[1] & 0x80))
-      buff[0]=(command==IDENTIFY)?WIN_PIDENTIFY:WIN_IDENTIFY;
+      buff[0]=(command==IDENTIFY)?ATA_IDENTIFY_PACKET_DEVICE:ATA_IDENTIFY_DEVICE;
   }
 #endif
   
@@ -400,9 +444,14 @@ int ata_command_interface(int device, smart_command_set command, int select, cha
   if ((retval=ioctl(device, HDIO_DRIVE_CMD, buff)))
     return -1;
 
-  // if the command returns data, copy it back
+  // CHECK POWER MODE command returns information in the Sector Count
+  // register (buff[3]).  Copy to return data buffer.
+  if (command==CHECK_POWER_MODE)
+    buff[HDIO_DRIVE_CMD_OFFSET]=buff[2];
+
+  // if the command returns data then copy it back
   if (copydata)
-    memcpy(data, buff+HDIO_DRIVE_CMD_OFFSET, 512);
+    memcpy(data, buff+HDIO_DRIVE_CMD_OFFSET, copydata);
   
   return 0; 
 }
@@ -602,7 +651,7 @@ int escalade_command_interface(int fd, int disknum, smart_command_set command, i
   passthru.cylinder_hi = 0xC2;
   
   // SMART ATA COMMAND REGISTER value
-  passthru.command = WIN_SMART;
+  passthru.command = ATA_SMART_CMD;
   
   // Is this a command that returns 512 bytes?
   if (command == READ_VALUES ||
@@ -628,20 +677,26 @@ int escalade_command_interface(int fd, int disknum, smart_command_set command, i
   
   // Now set ATA registers depending upon command
   switch (command){
+  case CHECK_POWER_MODE:
+    passthru.command  = ATA_CHECK_POWER_MODE;
+    passthru.features    = 0;
+    passthru.cylinder_lo = 0;
+    passthru.cylinder_hi = 0;
+    break;
   case READ_VALUES:
-    passthru.features = SMART_READ_VALUES;
+    passthru.features = ATA_SMART_READ_VALUES;
     break;
   case READ_THRESHOLDS:
-    passthru.features = SMART_READ_THRESHOLDS;
+    passthru.features = ATA_SMART_READ_THRESHOLDS;
     break;
   case READ_LOG:
-    passthru.features = SMART_READ_LOG_SECTOR;
+    passthru.features = ATA_SMART_READ_LOG_SECTOR;
     // log number to return
     passthru.sector_num  = select;
     break;
   case IDENTIFY:
     // ATA IDENTIFY DEVICE
-    passthru.command     = 0xEc;
+    passthru.command     = ATA_IDENTIFY_DEVICE;
     passthru.features    = 0;
     passthru.cylinder_lo = 0;
     passthru.cylinder_hi = 0;
@@ -652,38 +707,38 @@ int escalade_command_interface(int fd, int disknum, smart_command_set command, i
     errno=ENODEV;
     return -1;
   case ENABLE:
-    passthru.features = SMART_ENABLE;
+    passthru.features = ATA_SMART_ENABLE;
     break;
   case DISABLE:
-    passthru.features = SMART_DISABLE;
+    passthru.features = ATA_SMART_DISABLE;
     break;
   case AUTO_OFFLINE:
-    passthru.features = SMART_AUTO_OFFLINE;
+    passthru.features = ATA_SMART_AUTO_OFFLINE;
     // Enable or disable?
     passthru.sector_count = select;
     break;
   case AUTOSAVE:
-    passthru.features = SMART_AUTOSAVE;
+    passthru.features = ATA_SMART_AUTOSAVE;
     // Enable or disable?
     passthru.sector_count = select;
     break;
   case IMMEDIATE_OFFLINE:
-    passthru.features = SMART_IMMEDIATE_OFFLINE;
+    passthru.features = ATA_SMART_IMMEDIATE_OFFLINE;
     // What test type to run?
     passthru.sector_num  = select;
     break;
   case STATUS_CHECK:
-    passthru.features = SMART_STATUS;
+    passthru.features = ATA_SMART_STATUS;
     break;
   case STATUS:
     // This is JUST to see if SMART is enabled, by giving SMART status
     // command. But it doesn't say if status was good, or failing.
     // See below for the difference.
-    passthru.features = SMART_STATUS;
+    passthru.features = ATA_SMART_STATUS;
     break;
   default:
     pout("Unrecognized command %d in linux_3ware_command_interface(disk %d)\n"
-	 "Please contact " PACKAGE_BUGREPORT "\n", command, disknum);
+         "Please contact " PACKAGE_BUGREPORT "\n", command, disknum);
     errno=ENOSYS;
     return -1;
   }
@@ -713,12 +768,9 @@ int escalade_command_interface(int fd, int disknum, smart_command_set command, i
     memcpy(data, tw_output->output_data, 512);
   }
   
-  // We are finished with all commands except for STATUS_CHECK
-  if (command!=STATUS_CHECK) {
-    return 0;
-  }
-  else {
-
+  // For STATUS_CHECK, we need to check register values
+  if (command==STATUS_CHECK) {
+    
     // To find out if the SMART RETURN STATUS is good or failing, we
     // need to examine the values of the Cylinder Low and Cylinder
     // High Registers.
@@ -741,6 +793,22 @@ int escalade_command_interface(int fd, int disknum, smart_command_set command, i
     errno=ENOSYS;
     return 0;
   }
+
+  // copy sector count register (one byte!) to return data
+  if (command==CHECK_POWER_MODE) {
+    TW_Output *tw_output=(TW_Output *)&ioctlbuf;
+    TW_Passthru *tw_passthru_returned=(TW_Passthru *)&(tw_output->output_data);
+    *data=*(char *)&(tw_passthru_returned->sector_count);
+  }
+
+  // look for nonexistent devices/ports
+  if (command==IDENTIFY && !nonempty((unsigned char *)data, 512)) {
+    errno=ENODEV;
+    return -1;
+  }
+
+  return 0;
+
 }
 
 // Utility function for printing warnings
@@ -749,7 +817,7 @@ void printwarning(smart_command_set command){
   const char* message=
     "can not be passed through the 3ware 3w-xxxx driver.  This can be fixed by\n"
     "applying a simple 3w-xxxx driver patch that can be found here:\n"
-    PROJECTHOME "\n"
+    PACKAGE_HOMEPAGE "\n"
     "Alternatively, upgrade your 3w-xxxx driver to version 1.02.00.037 or greater.\n\n";
 
   if (command==AUTO_OFFLINE && !printed1) {
@@ -799,37 +867,37 @@ int guess_device_type(const char * dev_name) {
   
   // form /dev/h* or h*
   if (!strncmp(lin_dev_ata_disk_plus, dev_name,
-	       strlen(lin_dev_ata_disk_plus)))
+               strlen(lin_dev_ata_disk_plus)))
     return GUESS_DEVTYPE_ATA;
   
   // form /dev/ide/* or ide/*
   if (!strncmp(lin_dev_ata_devfs_disk_plus, dev_name,
-	       strlen(lin_dev_ata_devfs_disk_plus)))
+               strlen(lin_dev_ata_devfs_disk_plus)))
     return GUESS_DEVTYPE_ATA;
 
   // form /dev/s* or s*
   if (!strncmp(lin_dev_scsi_disk_plus, dev_name,
-	       strlen(lin_dev_scsi_disk_plus)))
+               strlen(lin_dev_scsi_disk_plus)))
     return GUESS_DEVTYPE_SCSI;
 
   // form /dev/scsi/* or scsi/*
   if (!strncmp(lin_dev_scsi_devfs_disk_plus, dev_name,
-	       strlen(lin_dev_scsi_devfs_disk_plus)))
+               strlen(lin_dev_scsi_devfs_disk_plus)))
     return GUESS_DEVTYPE_SCSI;
   
   // form /dev/ns* or ns*
   if (!strncmp(lin_dev_scsi_tape1, dev_name,
-	       strlen(lin_dev_scsi_tape1)))
+               strlen(lin_dev_scsi_tape1)))
     return GUESS_DEVTYPE_SCSI;
   
   // form /dev/os* or os*
   if (!strncmp(lin_dev_scsi_tape2, dev_name,
-	       strlen(lin_dev_scsi_tape2)))
+               strlen(lin_dev_scsi_tape2)))
     return GUESS_DEVTYPE_SCSI;
   
   // form /dev/nos* or nos*
   if (!strncmp(lin_dev_scsi_tape3, dev_name,
-	       strlen(lin_dev_scsi_tape3)))
+               strlen(lin_dev_scsi_tape3)))
     return GUESS_DEVTYPE_SCSI;
   
   // we failed to recognize any of the forms
