@@ -3,7 +3,7 @@
  *
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2002-3 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2002-4 Bruce Allen <smartmontools-support@lists.sourceforge.net>
  * Copyright (C) 2000 Michael Cornwell <cornwell@acm.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@
 #include "utility.h"
 
 extern const char *atacmdnames_c_cvsid, *atacmds_c_cvsid, *ataprint_c_cvsid, *knowndrives_c_cvsid, *os_XXXX_c_cvsid, *scsicmds_c_cvsid, *scsiprint_c_cvsid, *utility_c_cvsid; 
-const char* smartctl_c_cvsid="$Id: smartctl.c,v 1.106 2003/11/26 10:37:00 dpgilbert Exp $"
+const char* smartctl_c_cvsid="$Id: smartctl.c,v 1.114 2004/01/27 15:29:17 ballen4705 Exp $"
 ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID SCSICMDS_H_CVSID SCSIPRINT_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
 
 // This is a block containing all the "control variables".  We declare
@@ -57,14 +57,16 @@ extern int exitstatus;
 extern long long bytes;
 
 void printslogan(){
-  pout("smartctl version %s Copyright (C) 2002-3 Bruce Allen\n", PACKAGE_VERSION);
-  pout("Home page is %s\n\n",PROJECTHOME);
+  pout("smartctl version %s Copyright (C) 2002-4 Bruce Allen\n", PACKAGE_VERSION);
+  pout("Home page is " PACKAGE_HOMEPAGE "\n\n");
   return;
 }
 
 
 void printcopy(){
   char out[CVSMAXLEN];
+  char *configargs=strlen(SMARTMONTOOLS_CONFIGURE_ARGS)?SMARTMONTOOLS_CONFIGURE_ARGS:"[no arguments given]";
+
   pout("smartctl comes with ABSOLUTELY NO WARRANTY. This\n");
   pout("is free software, and you are welcome to redistribute it\n");
   pout("under the terms of the GNU General Public License Version 2.\n");
@@ -88,10 +90,11 @@ void printcopy(){
   pout("%s",out);
   printone(out,utility_c_cvsid);
   pout("%s",out);
-  pout("\nsmartctl build configured on " SMARTMONTOOLS_CONFIGURE_DATE "\n");
-  pout("smartctl configure arguments: " SMARTMONTOOLS_CONFIGURE_ARGS "\n");
-
-
+  pout("\nsmartmontools release " PACKAGE_VERSION " dated " SMARTMONTOOLS_RELEASE_DATE " at " SMARTMONTOOLS_RELEASE_TIME "\n");
+  pout("smartmontools build host: " SMARTMONTOOLS_BUILD_HOST "\n");
+  pout("smartmontools build configured: " SMARTMONTOOLS_CONFIGURE_DATE "\n");
+  pout("smartctl compile dated " __DATE__ " at "__TIME__ "\n");
+  pout("smartmontools configure arguments: %s\n", configargs);
   return;
 }
 
@@ -224,29 +227,8 @@ void Usage (void){
 "  -X        Abort any non-captive test\n\n"
   );
 #endif
-  printf("=================================================== SMARTCTL EXAMPLES =====\n\n");
-#ifdef HAVE_GETOPT_LONG
-  printf(
-"  smartctl -a /dev/hda                       (Prints all SMART information)\n\n"
-"  smartctl --smart=on --offlineauto=on --saveauto=on /dev/hda\n"
-"                                              (Enables SMART on first disk)\n\n"
-"  smartctl -t long /dev/hda              (Executes extended disk self-test)\n\n"
-"  smartctl --attributes --log=selftest --quietmode=errorsonly /dev/hda\n"
-"                                      (Prints Self-Test & Attribute errors)\n"
-"  smartctl -a -device=3ware,2 /dev/sda\n"
-"          (Prints all SMART info for 3rd ATA disk on 3ware RAID controller)\n"
-  );
-#else
-  printf(
-"  smartctl -a /dev/hda                       (Prints all SMART information)\n"
-"  smartctl -s on -o on -S on /dev/hda         (Enables SMART on first disk)\n"
-"  smartctl -t long /dev/hda              (Executes extended disk self-test)\n"
-"  smartctl -A -l selftest -q errorsonly /dev/hda\n"
-"                                      (Prints Self-Test & Attribute errors)\n"
-"  smartctl -a -d 3ware,2 /dev/sda\n"
-"          (Prints all SMART info for 3rd ATA disk on 3ware RAID controller)\n"
-  );
-#endif
+  print_smartctl_examples();
+  return;
 }
 
 /* Returns a pointer to a static string containing a formatted list of the valid
@@ -317,7 +299,7 @@ void printvalidarglistmessage(char opt) {
 
 unsigned char tryata=0,tryscsi=0;
 
-/*      Takes command options and sets features to be run */	
+/*      Takes command options and sets features to be run */    
 void ParseOpts (int argc, char** argv){
   int optchar;
   int badarg;
@@ -368,64 +350,64 @@ void ParseOpts (int argc, char** argv){
   // This miserable construction is needed to get emacs to do proper indenting. Sorry!
   while (-1 != (optchar = 
 #ifdef HAVE_GETOPT_LONG
-		getopt_long(argc, argv, shortopts, longopts, NULL)
+                getopt_long(argc, argv, shortopts, longopts, NULL)
 #else
-		getopt(argc, argv, shortopts)
+                getopt(argc, argv, shortopts)
 #endif
-		)){
+                )){
     switch (optchar){
     case 'V':
-      con->veryquietmode=FALSE;
+      con->dont_print=FALSE;
       printslogan();
       printcopy();
       exit(0);
       break;
     case 'q':
       if (!strcmp(optarg,"errorsonly")) {
-        con->quietmode     = TRUE;
-        con->veryquietmode = FALSE;
+        con->printing_switchable     = TRUE;
+        con->dont_print = FALSE;
       } else if (!strcmp(optarg,"silent")) {
-        con->veryquietmode = TRUE;
-        con->quietmode     = TRUE;
+        con->printing_switchable     = FALSE;
+        con->dont_print = TRUE;
       } else {
         badarg = TRUE;
       }
       break;
     case 'd':
       if (!strcmp(optarg,"ata")) {
-	tryata  = TRUE;
-	tryscsi = FALSE;
-	con->escalade = 0;
+        tryata  = TRUE;
+        tryscsi = FALSE;
+        con->escalade = 0;
       } else if (!strcmp(optarg,"scsi")) {
-	tryata  = FALSE;
-	tryscsi = TRUE;
-	con->escalade = 0;
+        tryata  = FALSE;
+        tryscsi = TRUE;
+        con->escalade = 0;
       } else {
-	// look for RAID-type device
-	int i;
+        // look for RAID-type device
+        int i;
         char *s;
-	
-	// make a copy of the string to mess with
-	if (!(s = strdup(optarg))) {
-          con->veryquietmode = FALSE;
+        
+        // make a copy of the string to mess with
+        if (!(s = strdup(optarg))) {
+          con->dont_print = FALSE;
           pout("No memory for argument of -d. Exiting...\n");
           exit(FAILCMD);
         } else if (strncmp(s,"3ware,",6)) {
-	  badarg = TRUE;
-	} else if (split_report_arg2(s, &i)) {
-	  pout("Option -d 3ware,N requires N to be a non-negative integer\n");
-	  badarg = TRUE;
+          badarg = TRUE;
+        } else if (split_report_arg2(s, &i)) {
+          pout("Option -d 3ware,N requires N to be a non-negative integer\n");
+          badarg = TRUE;
         } else if (i<0 || i>15) {
-	  pout("Option -d 3ware,N (N=%d) must have 0 <= N <= 15\n", i);
-	  badarg = TRUE;
-	} else {
-	  // NOTE: escalade = disk number + 1
+          pout("Option -d 3ware,N (N=%d) must have 0 <= N <= 15\n", i);
+          badarg = TRUE;
+        } else {
+          // NOTE: escalade = disk number + 1
           con->escalade = i+1;
-	  tryata  = TRUE;
-	  tryscsi = FALSE;
-	}
+          tryata  = TRUE;
+          tryscsi = FALSE;
+        }
         free(s);
-      } 	
+      }         
       break;
     case 'T':
       if (!strcmp(optarg,"normal")) {
@@ -435,7 +417,7 @@ void ParseOpts (int argc, char** argv){
         con->conservative = TRUE;
       } else if (!strcmp(optarg,"permissive")) {
         if (con->permissive<0xff)
-	  con->permissive++;
+          con->permissive++;
       } else if (!strcmp(optarg,"verypermissive")) {
         con->permissive=0xff;
       } else {
@@ -464,7 +446,7 @@ void ParseOpts (int argc, char** argv){
         // split_report_arg() may modify its first argument string, so use a
         // copy of optarg in case we want optarg for an error message.
         if (!(s = strdup(optarg))) {
-          con->veryquietmode = FALSE;
+          con->dont_print = FALSE;
           pout("Can't allocate memory to copy argument to -r option"
                " - exiting\n");
           EXIT(FAILCMD);
@@ -517,7 +499,7 @@ void ParseOpts (int argc, char** argv){
       }
       break;
     case 'H':
-      con->checksmart = TRUE;		
+      con->checksmart = TRUE;           
       break;
     case 'F':
       if (!strcmp(optarg,"none")) {
@@ -549,7 +531,7 @@ void ParseOpts (int argc, char** argv){
       break;
     case 'i':
       con->driveinfo = TRUE;
-      break;		
+      break;            
     case 'a':
       con->driveinfo          = TRUE;
       con->checksmart         = TRUE;
@@ -562,7 +544,7 @@ void ParseOpts (int argc, char** argv){
       // parse vendor-specific definitions of attributes
       if (!strcmp(optarg,"help")) {
         char *s;
-        con->veryquietmode=FALSE;
+        con->dont_print=FALSE;
         printslogan();
         if (!(s = create_vendor_attribute_arg_list())) {
           pout("Insufficient memory to construct argument list\n");
@@ -574,11 +556,11 @@ void ParseOpts (int argc, char** argv){
       }
       charp=con->attributedefs;
       if (!charp){
-	pout("Fatal internal error in ParseOpts()\n");
-	EXIT(FAILCMD);
+        pout("Fatal internal error in ParseOpts()\n");
+        EXIT(FAILCMD);
       }
       if (parse_attribute_def(optarg, &charp))
-	badarg = TRUE;
+        badarg = TRUE;
       break;    
     case 'P':
       if (!strcmp(optarg, "use")) {
@@ -615,7 +597,7 @@ void ParseOpts (int argc, char** argv){
           badarg = TRUE;
         } else {
           if (con->smartselectivenumspans >= 5 || start > stop) {
-            con->veryquietmode=FALSE;
+            con->dont_print=FALSE;
             printslogan();
             if (start > stop) {
               pout("ERROR: Start LBA > ending LBA in argument \"%s\"\n",
@@ -624,7 +606,7 @@ void ParseOpts (int argc, char** argv){
               pout("ERROR: No more than five selective self-test spans may be"
                 " defined\n");
             }
-	    UsageSummary();
+            UsageSummary();
             EXIT(FAILCMD);
           }
           con->smartselectivespan[con->smartselectivenumspans][0] = start;
@@ -645,14 +627,14 @@ void ParseOpts (int argc, char** argv){
       con->testcase           = ABORT_SELF_TEST;
       break;
     case 'h':
-      con->veryquietmode=FALSE;
+      con->dont_print=FALSE;
       printslogan();
       Usage();
-      EXIT(0);	
+      EXIT(0);  
       break;
     case '?':
     default:
-      con->veryquietmode=FALSE;
+      con->dont_print=FALSE;
       printslogan();
 #ifdef HAVE_GETOPT_LONG
       // Point arg to the argument in which this option was found.
@@ -663,10 +645,10 @@ void ParseOpts (int argc, char** argv){
         if (optopt && (strchr(shortopts, optopt) != NULL)) {
           pout("=======> ARGUMENT REQUIRED FOR OPTION: %s <=======\n", arg+2);
           printvalidarglistmessage(optopt);
-	} else
-	  pout("=======> UNRECOGNIZED OPTION: %s <=======\n",arg+2);
-	UsageSummary();
-	EXIT(FAILCMD);
+        } else
+          pout("=======> UNRECOGNIZED OPTION: %s <=======\n",arg+2);
+        UsageSummary();
+        EXIT(FAILCMD);
       }
 #endif
       if (optopt) {
@@ -677,12 +659,12 @@ void ParseOpts (int argc, char** argv){
           pout("=======> ARGUMENT REQUIRED FOR OPTION: %c <=======\n", optopt);
           printvalidarglistmessage(optopt);
         } else
-	  pout("=======> UNRECOGNIZED OPTION: %c <=======\n",optopt);
-	UsageSummary();
-	EXIT(FAILCMD);
+          pout("=======> UNRECOGNIZED OPTION: %c <=======\n",optopt);
+        UsageSummary();
+        EXIT(FAILCMD);
       }
       Usage();
-      EXIT(0);	
+      EXIT(0);  
     } // closes switch statement to process command-line options
     
     // Check to see if option had an unrecognized or incorrect argument.
@@ -697,16 +679,16 @@ void ParseOpts (int argc, char** argv){
       EXIT(FAILCMD);
     }
   }
-  // At this point we have processed all command-line options.
-
-  // Do this here, so results are independent of argument order	
-  if (con->quietmode)
-    con->veryquietmode=TRUE;
+  // At this point we have processed all command-line options.  If the
+  // print output is switchable, then start with the print output
+  // turned off
+  if (con->printing_switchable)
+    con->dont_print=TRUE;
 
   // error message if user has asked for more than one test
   if (1<(con->smartexeoffimmediate+con->smartshortselftest+con->smartextendselftest+
-	 con->smartshortcapselftest+con->smartextendcapselftest+con->smartselftestabort)){
-    con->veryquietmode=FALSE;
+         con->smartshortcapselftest+con->smartextendcapselftest+con->smartselftestabort)){
+    con->dont_print=FALSE;
     printslogan();
     pout("\nERROR: smartctl can only run a single test (or abort) at a time.\n");
     UsageSummary();
@@ -758,7 +740,7 @@ void ParseOpts (int argc, char** argv){
   }  
 }
 
-// Printing function (controlled by global con->veryquietmode) 
+// Printing function (controlled by global con->dont_print) 
 // [From GLIBC Manual: Since the prototype doesn't specify types for
 // optional arguments, in a call to a variadic function the default
 // argument promotions are performed on the optional argument
@@ -770,7 +752,7 @@ void pout(char *fmt, ...){
   
   // initialize variable argument list 
   va_start(ap,fmt);
-  if (con->veryquietmode){
+  if (con->dont_print){
     va_end(ap);
     return;
   }
